@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile::Schema::Instance;
 use vars '$VERSION';
-$VERSION = '0.14';
+$VERSION = '0.15';
 
 use Carp;
 
@@ -64,10 +64,10 @@ sub substitutionGroupMembers($)
 
 
 my %as_element = map { ($_ => 1) }
-   qw/element group attributeGroup/;
+   qw/element group attributeGroup attribute/;
 
 my %as_type    = map { ($_ => 1) }
-   qw/complexType simpleType attribute attributeGroup group/;
+   qw/complexType simpleType attributeGroup group/;
 
 my %skip_toplevel = map { ($_ => 1) }
    qw/annotation import notation include redefine/;
